import CPU from './cpu';
import Keyboard from './keyboard'
import RNG from './RNG'

class Chip8Emulator {
	cpu: CPU;
	keyboard: Keyboard;
	rom: Uint8Array;

	romLoaded: boolean;

	constructor() {
		this.keyboard = new Keyboard();
		this.cpu = new CPU(this.keyboard);
		
		this.rom = new Uint8Array();
		this.romLoaded = false;
	}

	loadRom(rom: Uint8Array) {
		this.rom = rom;
		this.cpu.loadRom(this.rom);
		this.romLoaded = true;
	}

	isRomLoaded() {
		return this.romLoaded;
	}

	emulateCycle() {
		this.cpu.cycle();
	}

	getCPUStatus() : string {
		return this.cpu.getCPUStatus();
	}

	drawToCanvas(canvas: HTMLCanvasElement, size: number) {
		let ctx = canvas.getContext('2d') as CanvasRenderingContext2D;
		let width = 64*size;
		let height = 32*size;

		// Clear screen
		ctx.fillStyle = "BLACK";
		ctx.fillRect(0, 0, width, height);

		// Draw rectangles as pixels
		for(let i = 0; i < this.cpu.displayMemory.length; i++) {
			let isOn: number = this.cpu.displayMemory[i];
			if(isOn === 1) {
				let y = Math.floor(i/64);
				let x = i - (y*64);

				ctx.fillStyle = "WHITE";
				ctx.fillRect(x*size, y*size, size, size);
			}
		}
	}

	keyDownEvent(key: number) {
		this.keyboard.userKeyPressedDown(key);
	}

	keyUpEvent(key: number) {
		this.keyboard.userKeyPressedUp(key);
	}



}

export default Chip8Emulator;